from pathlib import Path
import os.path
from typing import List, Optional
import pytest
from pytest_steps import test_steps

import api_debug
import pwndoc_api
from api_process_findings import ProcessingSettings, ProcessingStatus, upload_scan_findings_to_pwndoc
from tests.helpers_for_test import pwndoc_connectivity_test
from tests.helper_anotate_tests import skip_and_anotate_problematic_test_files
from tests.conftest import TMP_PATH, PWNDOC_DANGER_OVERRIDE
import config


def find_nessus_test_files() -> List[str]:
    answer = [str(x) for x in Path('tests/test_files').rglob('*.nessus')]
    answer = skip_and_anotate_problematic_test_files(answer)
    return answer


def find_burp_test_files() -> List[str]:
    answer = [str(x) for x in Path('tests/test_files').rglob('*/burp/*.xml')]
    answer = skip_and_anotate_problematic_test_files(answer)
    return answer


def find_lynis_test_files() -> List[str]:
    answer = [str(x) for x in Path('tests/test_files').rglob('*-lynis-*.dat')]
    answer = skip_and_anotate_problematic_test_files(answer)
    return answer


def find_all_test_files() -> List[str]:
    return find_nessus_test_files() + find_burp_test_files() + find_lynis_test_files()


@pytest.mark.dependency()
def test_connectivity():
    pwndoc_connectivity_test()


@pytest.mark.skipif(not PWNDOC_DANGER_OVERRIDE, reason="PWNDOC_DANGER_OVERRIDE must be allowed to enable testing of interaction with PwnDoc")
@pytest.mark.dependency(depends=["test_connectivity"])
class TestFindingParsing:

    @test_steps('PwnDoc Templates, Scan2Report, JSON', 'Upload to PwnDoc Audit', 'Download PwnDoc audit report')
    @pytest.mark.parametrize("filepath", find_all_test_files())
    def test_full(self, client, filepath):
        test_filename = os.path.basename(filepath)
        folder_name = self.to_json(client, filepath)
        yield

        self.upload_to_pwndoc(folder_name, test_filename)
        yield

        self.download_report(folder_name, test_filename)
        yield

    def to_json(self, client, filepath: str) -> str:
        response = client.post(
            '/import_automator/findings/upload_scanner_result',
            follow_redirects=True,
            data={
                'audit': 'json',
                'locale': 'cs',
                'profile': 'default',
                'min-severity': '0',
                'scan2report_args': '',
                'file[]': open(filepath, 'rb'),
            },
        )
        assert response.status_code == 200, f'The response code indicates uncaught failure. Status code: {response.status_code}'

        filename = os.path.basename(filepath)
        with open(os.path.join(TMP_PATH, f'{filename}.html'), 'w', encoding='utf8') as f:
            f.write(response.text)

        url_path = response.request.path
        _, folder_name = url_path.rstrip("/").rsplit("/", 1)
        # status_filepath = os.path.join(config.FLASK_UPLOAD_FOLDER, folder_id, '_pwndoc_importer_processing.json')
        # status = helpers.file_utils.json_safe_load(status_filepath)
        ps = ProcessingSettings.load_from_folder_name(folder_name)
        assert ps is not None, "Report of import should exist"

        for step_name, (step_status, _, _) in ps.steps_progress.items():
            assert step_status != ProcessingStatus.FAILED.name, f"Step {step_name} failed."

        for level, msg in ps.user_msgs:
            assert level != 'error', f'There is at least one error msg displayed to user: {msg}'

        return folder_name

    def upload_to_pwndoc(self, folder_name: str, test_filename: str = ""):
        def _safe_name(dangerous_name: str) -> str:
            return ''.join(x for x in dangerous_name if x.isalnum())[:30]

        audit_id = pwndoc_api.create_audit(name_section=_safe_name(test_filename))
        ps = ProcessingSettings.load_from_folder_name(folder_name)
        assert ps is not None

        ps.upload_to_pwndoc = True
        ps.audit_id = audit_id
        ps.save()

        assert upload_scan_findings_to_pwndoc(folder_name)

    def download_report(self, folder_name: str, test_filename: str):
        ps = ProcessingSettings.load_from_folder_name(folder_name)
        report_filepath = os.path.abspath(os.path.join(TMP_PATH, f'{test_filename}.docx'))
        pwndoc_api.download_report(ps.audit_id, report_filepath)
        assert Path(report_filepath).is_file()
